"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getContentType = exports.getMethodParameters = exports.getResponseObject = exports.definitions = void 0;
const ts_morph_1 = require("ts-morph");
const typescript_1 = require("./typescript");
const lodash_1 = require("lodash");
const decorators_1 = require("./decorators");
exports.definitions = {};
function getResponseObject(responseType) {
    if (!responseType)
        return { description: "ok" };
    if (responseType.getText().includes("Promise"))
        responseType = responseType === null || responseType === void 0 ? void 0 : responseType.getTypeArguments()[0];
    const responseObject = {};
    const contentType = getContentType(responseType);
    responseObject[contentType] = { schema: getParamSchema(responseType) };
    return { content: responseObject, description: "ok" };
}
exports.getResponseObject = getResponseObject;
function getMethodParameters(parameters) {
    const result = {
        requestBody: undefined,
        parameters: [],
    };
    parameters.forEach((parameter) => {
        parameter
            .getDecorators()
            .filter((d) => ["Body", "Param", "Query", "Headers"].includes(d.getName()))
            .forEach((decorator) => {
            var _a, _b;
            const decoratorName = decorator.getName();
            const decoratorArgs = decorator.getArguments();
            const inputPath = (_a = decoratorArgs[0]) === null || _a === void 0 ? void 0 : _a.getLiteralValue();
            switch (decoratorName) {
                case "Body": {
                    const contentType = getContentType(parameter.getType());
                    if (inputPath) {
                        result.requestBody = result.requestBody || { content: { "application/json": { schema: { type: "object" } } } };
                        const propSchema = getParamSchema(parameter);
                        const schema = result.requestBody.content["application/json"].schema;
                        schema.properties = schema.properties || {};
                        if (propSchema)
                            schema.properties[inputPath] = propSchema;
                        if (!((_b = propSchema) === null || _b === void 0 ? void 0 : _b.optional))
                            schema.required = (schema.required || []).concat(inputPath);
                    }
                    else {
                        result.requestBody = { content: {} };
                        result.requestBody.content[contentType] = { schema: getParamSchema(parameter) };
                    }
                    break;
                }
                case "Param": {
                    if (inputPath) {
                        const paramSchema = getParamSchema(parameter);
                        result.parameters.push({
                            in: "path",
                            name: inputPath,
                            required: !paramSchema.optional,
                            schema: paramSchema,
                        });
                    }
                    else {
                        const objectSchema = getParamSchema(parameter, false);
                        Object.keys(objectSchema.properties || {}).forEach((prop) => {
                            var _a;
                            const propSchema = (_a = objectSchema.properties) === null || _a === void 0 ? void 0 : _a[prop];
                            result.parameters.push({
                                in: "path",
                                name: prop,
                                required: !propSchema.optional,
                                schema: propSchema,
                            });
                        });
                    }
                    break;
                }
                case "Query": {
                    if (inputPath) {
                        const paramSchema = getParamSchema(parameter);
                        result.parameters.push({
                            in: "query",
                            name: inputPath,
                            required: !paramSchema.optional,
                            schema: paramSchema,
                        });
                    }
                    else {
                        const paramSchema = getParamSchema(parameter, false);
                        Object.keys(paramSchema.properties || {}).forEach((prop) => {
                            var _a;
                            const propSchema = (_a = paramSchema.properties) === null || _a === void 0 ? void 0 : _a[prop];
                            result.parameters.push({
                                in: "query",
                                name: prop,
                                required: !propSchema.optional,
                                schema: propSchema,
                            });
                        });
                    }
                    break;
                }
                case "Headers": {
                    if (inputPath) {
                        const paramSchema = getParamSchema(parameter);
                        result.parameters.push({
                            in: "header",
                            name: inputPath,
                            required: !paramSchema.optional,
                            schema: paramSchema,
                        });
                    }
                    else {
                        const paramSchema = getParamSchema(parameter, false);
                        Object.keys(paramSchema.properties || {}).forEach((prop) => {
                            var _a;
                            const propSchema = (_a = paramSchema.properties) === null || _a === void 0 ? void 0 : _a[prop];
                            result.parameters.push({
                                in: "header",
                                name: prop,
                                required: !propSchema.optional,
                                schema: propSchema,
                            });
                        });
                    }
                    break;
                }
            }
        });
    });
    return result;
}
exports.getMethodParameters = getMethodParameters;
function getContentType(type) {
    if ((0, typescript_1.isPrimitive)(type))
        return "text/plain";
    return "application/json";
}
exports.getContentType = getContentType;
const getObjectSchema = (type) => {
    var _a;
    const schema = { type: "object", properties: {}, required: [] };
    type.getProperties().forEach((prop) => {
        const isGetter = prop.hasFlags(ts_morph_1.SymbolFlags.GetAccessor);
        if (isGetter)
            return;
        const valueDeclaration = prop.getValueDeclarationOrThrow();
        const schemaProperties = schema.properties;
        const key = prop.getName();
        schemaProperties[key] = getParamSchema(valueDeclaration) || {};
        if (!schemaProperties[key]) {
            schemaProperties[key] = { type: "object" };
        }
        if (!schemaProperties[key].optional) {
            schema === null || schema === void 0 ? void 0 : schema.required.push(key);
        }
    });
    if (!((_a = schema.required) === null || _a === void 0 ? void 0 : _a.length))
        delete schema.required;
    return schema;
};
const getTypeString = (type, useRef) => {
    const typeText = type.getText();
    if (type.isArray())
        return "array";
    if (type.getText() === "Date")
        return "date";
    if ((0, typescript_1.isPrimitive)(type))
        return typeText.replace(" | undefined", "");
    if (type.isClass() || type.isInterface())
        return useRef ? "ref" : "object";
    if (type.isObject())
        return "object";
    if (type.isEnumLiteral())
        return "enumLiteral";
    if (type.isEnum())
        return "enum";
};
const getParamSchema = (typeOrProperty, useRef = true) => {
    const type = typeOrProperty instanceof ts_morph_1.Type ? typeOrProperty : typeOrProperty.getType();
    const prop = typeOrProperty instanceof ts_morph_1.Type ? undefined : typeOrProperty;
    const typeText = type.getText();
    const nonNullableType = type.getNonNullableType();
    const typeString = getTypeString(nonNullableType, useRef);
    let schema = { optional: nonNullableType.getText() !== typeText || !!(prop === null || prop === void 0 ? void 0 : prop.hasQuestionToken()) };
    if (prop instanceof ts_morph_1.PropertyDeclaration || prop instanceof ts_morph_1.ParameterDeclaration) {
        prop.getDecorators().forEach((decorator) => {
            schema = (0, lodash_1.merge)(schema, (0, decorators_1.getCustomValidation)(decorator, typeString));
        });
    }
    switch (typeString) {
        case "array":
            return Object.assign(Object.assign({}, schema), { type: "array", items: getParamSchema(nonNullableType.getArrayElementTypeOrThrow()) || {} });
        case "date":
            return Object.assign(Object.assign({}, schema), { type: "string", format: "date-time" });
        case "boolean":
        case "number":
        case "string":
            return Object.assign({ type: typeString }, schema);
        case "ref": {
            const name = nonNullableType.getText().split(").")[1] || nonNullableType.getText();
            if (!exports.definitions[name])
                exports.definitions[name] = (0, lodash_1.merge)(getObjectSchema(nonNullableType), schema);
            return { $ref: "#/components/schemas/" + name };
        }
        case "object":
            return (0, lodash_1.merge)(getObjectSchema(nonNullableType), schema);
        case "enumLiteral": {
            const name = prop.getName();
            const enumMembers = prop
                .getValueDeclarationOrThrow()
                .getSourceFile()
                .getEnum((e) => e.getName() === nonNullableType.getText())
                .getMembers();
            const enumSchema = {};
            enumSchema.enum = enumMembers.map((m) => m.getName());
            enumSchema["x-enumNames"] = enumMembers.map((m) => m.getValue());
            enumSchema.type = typeof enumSchema.enum[0];
            exports.definitions[name] = enumSchema;
            return Object.assign({ $ref: "#/components/schemas/" + name }, schema);
        }
        case "enum": {
            const name = (0, lodash_1.last)(nonNullableType.getText().split("."));
            const enumSchema = {};
            enumSchema.enum = nonNullableType.getUnionTypes().map((t) => t.getLiteralValueOrThrow());
            enumSchema["x-enumNames"] = nonNullableType.getUnionTypes().map((t) => (0, lodash_1.last)(t.getText().split(".")));
            enumSchema.type = typeof enumSchema.enum[0];
            exports.definitions[name] = enumSchema;
            return Object.assign({ $ref: "#/components/schemas/" + name }, schema);
        }
    }
    const unionTypes = type.getUnionTypes().filter((t) => !t.isUndefined());
    if (unionTypes.length > 1) {
        const schema = {};
        schema.oneOf = unionTypes.map((t) => getParamSchema(t));
        if (!schema.oneOf[0]) {
            delete schema.oneOf;
            schema.enum = unionTypes.map((t) => t.getText().slice(1, -1));
            schema["x-enumNames"] = unionTypes.map((t) => t.getText().slice(1, -1));
            schema.type = typeof schema.enum[0];
        }
        return schema;
    }
};
